from django.template import TemplateDoesNotExist
import pytest
from collections.abc import Iterable

from django.contrib.auth.models import User
from django.urls import reverse
from django.utils import timezone
from django.test.utils import override_settings

from djpress.models import Category, Post
from djpress.utils import validate_date


@pytest.fixture
def user():
    user = User.objects.create_user(
        username="testuser",
        password="testpass",
    )
    return user


@pytest.fixture
def category():
    category = Category.objects.create(
        title="Test Category",
        slug="test-category",
    )
    return category


@pytest.fixture
def test_post1(user, category):
    post = Post.post_objects.create(
        title="Test Post",
        slug="test-post",
        content="This is a test post.",
        author=user,
        status="published",
        post_type="post",
        date=timezone.make_aware(timezone.datetime(2024, 1, 1)),
    )
    post.categories.set([category])
    return post


@pytest.fixture
def test_page1(user):
    page = Post.post_objects.create(
        title="Test Page",
        slug="test-page",
        content="This is a test page.",
        author=user,
        status="published",
        post_type="page",
    )
    return page


@pytest.mark.django_db
def test_index_view(client):
    url = reverse("djpress:index")
    response = client.get(url)
    assert response.status_code == 200
    assert b"No posts available" in response.content
    assert "posts" in response.context
    assert isinstance(response.context["posts"], Iterable)


@pytest.mark.django_db
def test_post_detail_view(client, test_post1, test_page1):
    # Test 1 - post
    url = reverse("djpress:post_detail", args=[test_post1.permalink])
    response = client.get(url)
    assert response.status_code == 200
    assert "post" in response.context
    assert not isinstance(response.context["post"], Iterable)

    # Test 2 - page
    url = reverse("djpress:post_detail", args=[test_page1.permalink])
    response = client.get(url)
    assert response.status_code == 200
    assert "post" in response.context
    assert not isinstance(response.context["post"], Iterable)


@pytest.mark.django_db
def test_post_detail_not_exist(client):
    url = reverse("djpress:post_detail", args=["foobar-does-not-exist"])
    response = client.get(url)
    assert response.status_code == 404


@pytest.mark.django_db
def test_author_with_no_posts_view(client, user):
    url = reverse("djpress:author_posts", args=[user.username])
    response = client.get(url)
    assert response.status_code == 200
    assert "author" in response.context
    assert b"No posts available" in response.content
    assert "posts" in response.context
    assert isinstance(response.context["posts"], Iterable)


@pytest.mark.django_db
def test_author_with_posts_view(client, test_post1):
    url = reverse("djpress:author_posts", args=[test_post1.author.username])
    response = client.get(url)
    assert response.status_code == 200
    assert "author" in response.context
    assert not b"No posts available" in response.content
    assert "posts" in response.context
    assert isinstance(response.context["posts"], Iterable)


@pytest.mark.django_db
def test_author_with_invalid_author(client):
    url = reverse("djpress:author_posts", args=["non-existent-author"])
    response = client.get(url)
    assert response.status_code == 404


@pytest.mark.django_db
def test_category_with_no_posts_view(client, category):
    url = reverse("djpress:category_posts", args=[category.slug])
    response = client.get(url)
    assert response.status_code == 200
    assert "category" in response.context
    assert b"No posts available" in response.content
    assert "posts" in response.context
    assert isinstance(response.context["posts"], Iterable)


@pytest.mark.django_db
def test_category_with_posts_view(client, test_post1, category):
    url = reverse("djpress:category_posts", args=[category.slug])
    response = client.get(url)
    assert response.status_code == 200
    assert "category" in response.context
    assert not b"No posts available" in response.content
    assert "posts" in response.context
    assert isinstance(response.context["posts"], Iterable)


@pytest.mark.django_db
def test_category_with_invalid_category(client):
    url = reverse("djpress:category_posts", args=["non-existent-category"])
    response = client.get(url)
    assert response.status_code == 404


def test_validate_date():
    # Test 1 - invalid year
    year = "0000"
    with pytest.raises(ValueError):
        validate_date(year, "", "")

    # Test 2 - invalid months
    month = "00"
    with pytest.raises(ValueError):
        validate_date("2024", month, "")
    month = "13"
    with pytest.raises(ValueError):
        validate_date("2024", month, "")

    # Test 3 - invalid days
    day = "00"
    with pytest.raises(ValueError):
        validate_date("2024", "1", day)
    day = "32"
    with pytest.raises(ValueError):
        validate_date("2024", "1", day)


@pytest.mark.django_db
def test_date_archives_year(client, test_post1):
    url = reverse("djpress:archives_posts", kwargs={"year": "2024"})
    response = client.get(url)
    assert response.status_code == 200
    assert test_post1.title.encode() in response.content
    assert "posts" in response.context
    assert isinstance(response.context["posts"], Iterable)


@pytest.mark.django_db
def test_date_archives_year_invalid_year(client):
    response = client.get("/archives/0000/")
    assert response.status_code == 400


@pytest.mark.django_db
def test_date_archives_year_no_posts(client, test_post1):
    url = reverse("djpress:archives_posts", kwargs={"year": "2023"})
    response = client.get(url)
    assert response.status_code == 200
    assert not test_post1.title.encode() in response.content
    assert b"No posts available" in response.content
    assert "posts" in response.context
    assert isinstance(response.context["posts"], Iterable)


@pytest.mark.django_db
def test_date_archives_month(client, test_post1):
    url = reverse("djpress:archives_posts", kwargs={"year": "2024", "month": "01"})
    response = client.get(url)
    assert response.status_code == 200
    assert test_post1.title.encode() in response.content
    assert "posts" in response.context
    assert isinstance(response.context["posts"], Iterable)


@pytest.mark.django_db
def test_date_archives_month_invalid_month(client):
    response1 = client.get("/archives/2024/00/")
    assert response1.status_code == 400
    response2 = client.get("/archives/2024/13/")
    assert response2.status_code == 400


@pytest.mark.django_db
def test_date_archives_month_no_posts(client, test_post1):
    url = reverse("djpress:archives_posts", kwargs={"year": "2024", "month": "02"})
    response = client.get(url)
    assert response.status_code == 200
    assert not test_post1.title.encode() in response.content
    assert b"No posts available" in response.content
    assert "posts" in response.context
    assert isinstance(response.context["posts"], Iterable)


@pytest.mark.django_db
def test_date_archives_day(client, test_post1):
    url = reverse(
        "djpress:archives_posts", kwargs={"year": "2024", "month": "01", "day": "01"}
    )
    response = client.get(url)
    assert response.status_code == 200
    assert "posts" in response.context
    assert test_post1.title.encode() in response.content


@pytest.mark.django_db
def test_date_archives_day_invalid_day(client):
    response1 = client.get("/archives/2024/01/00/")
    assert response1.status_code == 400
    response2 = client.get("/archives/2024/01/32/")
    assert response2.status_code == 400


@pytest.mark.django_db
def test_date_archives_day_no_posts(client, test_post1):
    url = reverse(
        "djpress:archives_posts", kwargs={"year": "2024", "month": "01", "day": "02"}
    )
    response = client.get(url)
    assert response.status_code == 200
    assert not test_post1.title.encode() in response.content
    assert b"No posts available" in response.content
    assert "posts" in response.context
    assert isinstance(response.context["posts"], Iterable)



"""Settings file for DJ Press."""

# DJPress settings
TRUNCATE_TAG = "<!--more-->"
CACHE_CATEGORIES: bool = True
CACHE_RECENT_PUBLISHED_POSTS: bool = False
RECENT_PUBLISHED_POSTS_COUNT: int = 20
MARKDOWN_EXTENSIONS: list = []
MARKDOWN_EXTENSION_CONFIGS: dict = {}
BLOG_TITLE: str = "My DJ Press Blog"
BLOG_DESCRIPTION: str = ""
POST_READ_MORE_TEXT: str = "Read more..."

# DJPress URL settings
CATEGORY_PATH_ENABLED: bool = True
CATEGORY_PATH: str = "category"
AUTHOR_PATH_ENABLED: bool = True
AUTHOR_PATH: str = "author"
ARCHIVES_PATH_ENABLED: bool = True
ARCHIVES_PATH: str = "archives"
DATE_ARCHIVES_ENABLED: bool = True
RSS_ENABLED: bool = True
RSS_PATH: str = "rss"

# The following are used to generate the post permalink
DAY_SLUG: str = "%Y/%m/%d"
MONTH_SLUG: str = "%Y/%m"
YEAR_SLUG: str = "%Y"
POST_PREFIX: str = "post"
POST_PERMALINK: str = ""



import pytest
from django.contrib.auth.models import User
from django.template import Context
from django.urls import reverse
from django.utils import timezone
from django.core.paginator import Paginator

from djpress.conf import settings
from djpress.models import Category, Post
from djpress.templatetags import djpress_tags
from djpress.templatetags.helpers import (
    post_read_more_link,
    categories_html,
    get_page_link,
)
from djpress.utils import get_author_display_name


@pytest.fixture
def user():
    user = User.objects.create_user(
        username="testuser",
        password="testpass",
        first_name="Test",
        last_name="User",
    )
    return user


@pytest.fixture
def category1():
    category = Category.objects.create(
        title="General",
        slug="general",
    )
    return category


@pytest.fixture
def category2():
    category = Category.objects.create(
        title="News",
        slug="news",
    )
    return category


@pytest.fixture
def category3():
    category = Category.objects.create(
        title="Development",
        slug="dev",
    )
    return category


@pytest.fixture
def test_post1(user, category1):
    post = Post.post_objects.create(
        title="Test Post1",
        slug="test-post1",
        content="This is a test post.",
        author=user,
        status="published",
        post_type="post",
    )
    post.categories.set([category1])
    return post


@pytest.fixture
def test_post2(user, category2):
    post = Post.post_objects.create(
        title="Test Post2",
        slug="test-post2",
        content="This is a test post.",
        author=user,
        status="published",
        post_type="post",
    )
    post.categories.set([category2])
    return post


@pytest.fixture
def test_long_post1(user, category1):
    post = Post.post_objects.create(
        title="Test Long Post1",
        slug="test-long-post1",
        content=f"This is the truncated content.\n\n{settings.TRUNCATE_TAG}\n\nThis is the rest of the post.",
        author=user,
        status="published",
        post_type="post",
    )
    post.categories.set([category1])
    return post


@pytest.fixture
def test_page1(user):
    post = Post.post_objects.create(
        title="Test Page1",
        slug="test-page1",
        content="This is a test page.",
        author=user,
        status="published",
        post_type="page",
    )
    return post


@pytest.fixture
def test_page2(user):
    post = Post.post_objects.create(
        title="Test Page2",
        slug="test-page2",
        content="This is a test page.",
        author=user,
        status="published",
        post_type="page",
    )
    return post


@pytest.mark.django_db
def test_have_posts_single_post(test_post1):
    """Return a list of posts in the context."""
    context = Context({"post": test_post1})

    assert djpress_tags.have_posts(context) == [test_post1]


@pytest.mark.django_db
def test_have_posts_no_posts():
    """Return an empty list if there are no posts in the context."""
    context = Context({"foo": "bar"})

    assert djpress_tags.have_posts(context) == []


@pytest.mark.django_db
def test_have_posts_multiple_posts(test_post1, test_long_post1):
    """Return a list of posts in the context."""
    context = Context({"posts": [test_post1, test_long_post1]})

    assert djpress_tags.have_posts(context) == [test_post1, test_long_post1]


def test_blog_title():
    """Test the blog_title template tag.

    This can be changed on the fly.
    """
    assert settings.BLOG_TITLE == "My Test DJ Press Blog"
    assert djpress_tags.blog_title() == settings.BLOG_TITLE

    # Change the title
    settings.set("BLOG_TITLE", "My New Blog Title")
    assert settings.BLOG_TITLE == "My New Blog Title"
    assert djpress_tags.blog_title() == settings.BLOG_TITLE

    # Set the title back to the original
    settings.set("BLOG_TITLE", "My Test DJ Press Blog")
    assert settings.BLOG_TITLE == "My Test DJ Press Blog"
    assert djpress_tags.blog_title() == settings.BLOG_TITLE


@pytest.mark.django_db
def test_get_categories(category1, category2, category3):
    categories = Category.objects.all().order_by("menu_order").order_by("title")
    djpress_categories = djpress_tags.get_categories()

    assert category1 in categories
    assert category2 in categories
    assert category3 in categories
    assert category1 in djpress_categories
    assert category2 in djpress_categories
    assert category3 in djpress_categories

    assert list(djpress_categories) == list(categories)  # type: ignore


@pytest.mark.django_db
def test_post_title_single_post(test_post1):
    context = Context({"post": test_post1})
    assert djpress_tags.post_title(context) == test_post1.title


def test_post_title_no_post_context():
    context = Context({"foo": "bar"})
    assert djpress_tags.post_title(context) == ""
    assert type(djpress_tags.post_title(context)) == str


@pytest.mark.django_db
def test_post_title_posts(test_post1):
    """Test the post_title_link template tag.

    This uses the post.permalink property to generate the link."""
    # Context should have both a posts and a post to simulate the for post in posts loop
    context = Context({"posts": [test_post1], "post": test_post1})

    # Confirm settings in settings_testing.py
    assert settings.POST_PREFIX == "test-posts"

    # this generates a URL based on the slug only - this is prefixed with the POST_PREFIX setting
    post_url = reverse("djpress:post_detail", args=[test_post1.slug])

    # Confirm settings in settings_testing.py
    assert settings.POST_PREFIX == "test-posts"
    expected_output = f'<a href="/test-posts{post_url}" title="{test_post1.title}">{test_post1.title}</a>'
    assert djpress_tags.post_title_link(context) == expected_output


@pytest.mark.django_db
def test_post_title_link_no_context():
    context = Context()

    expected_output = ""
    assert djpress_tags.post_title_link(context) == expected_output


@pytest.mark.django_db
def test_post_title_link_with_prefix(test_post1):
    # Confirm settings in settings_testing.py
    assert settings.POST_PREFIX == "test-posts"

    # Context should have both a posts and a post to simulate the for post in posts loop
    context = Context({"posts": [test_post1], "post": test_post1})

    post_url = reverse("djpress:post_detail", args=[test_post1.slug])

    expected_output = f'<a href="/test-posts{post_url}" title="{test_post1.title}">{test_post1.title}</a>'
    assert djpress_tags.post_title_link(context) == expected_output


@pytest.mark.django_db
def test_post_author(test_post1):
    context = Context({"post": test_post1})

    author = test_post1.author
    output = f'<span rel="author">{ get_author_display_name(author) }</span>'
    assert djpress_tags.post_author(context) == output


def test_post_author_no_post():
    context = Context({"foo": "bar"})

    assert djpress_tags.post_author(context) == ""
    assert type(djpress_tags.post_author(context)) == str


def test_post_author_link_no_post():
    context = Context({"foo": "bar"})

    assert djpress_tags.post_author_link(context) == ""
    assert type(djpress_tags.post_author_link(context)) == str


@pytest.mark.django_db
def test_post_author_link(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.AUTHOR_PATH_ENABLED is True
    assert settings.AUTHOR_PATH == "test-url-author"

    author = test_post1.author

    expected_output = (
        f'<a href="/{settings.AUTHOR_PATH}/testuser/" title="View all posts by '
        f'{ get_author_display_name(author) }"><span rel="author">'
        f"{ get_author_display_name(author) }</span></a>"
    )
    assert djpress_tags.post_author_link(context) == expected_output


@pytest.mark.django_db
def test_post_author_link_author_path_disabled(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.AUTHOR_PATH_ENABLED is True
    assert settings.AUTHOR_PATH == "test-url-author"

    settings.set("AUTHOR_PATH_ENABLED", False)

    author = test_post1.author

    expected_output = f'<span rel="author">{get_author_display_name(author)}</span>'
    assert djpress_tags.post_author_link(context) == expected_output

    # Set back to defaults
    settings.set("AUTHOR_PATH_ENABLED", True)


@pytest.mark.django_db
def test_post_author_link_with_author_path_with_one_link_class(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.AUTHOR_PATH_ENABLED is True
    assert settings.AUTHOR_PATH == "test-url-author"

    author = test_post1.author

    expected_output = (
        f'<a href="/{settings.AUTHOR_PATH}/testuser/" title="View all posts by '
        f'{ get_author_display_name(author) }" class="class1">'
        f'<span rel="author">{ get_author_display_name(author) }</span></a>'
    )
    assert djpress_tags.post_author_link(context, "class1") == expected_output


@pytest.mark.django_db
def test_post_author_link_with_author_path_with_two_link_class(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.AUTHOR_PATH_ENABLED is True
    assert settings.AUTHOR_PATH == "test-url-author"

    author = test_post1.author

    expected_output = (
        f'<a href="/{settings.AUTHOR_PATH}/testuser/" title="View all posts by '
        f'{ get_author_display_name(author) }" class="class1 class2">'
        f'<span rel="author">{ get_author_display_name(author) }</span></a>'
    )
    assert djpress_tags.post_author_link(context, "class1 class2") == expected_output


@pytest.mark.django_db
def test_post_category_link_without_category_path(category1):
    """Test the post_category_link template tag without the category path enabled.

    If the CATEGORY_PATH_ENABLED setting is False, the template tag should just return
    the category name, with no link."""
    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH_ENABLED is True

    settings.set("CATEGORY_PATH_ENABLED", False)
    assert settings.CATEGORY_PATH_ENABLED is False

    assert djpress_tags.post_category_link(category1) == category1.title

    # Set back to defaults
    settings.set("CATEGORY_PATH_ENABLED", True)


@pytest.mark.django_db
def test_post_category_link_without_category_pathwith_one_link(category1):
    """Test the post_category_link template tag without the category path enabled.

    If the CATEGORY_PATH_ENABLED setting is False, the template tag should just return
    the category name, with no link."""
    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH_ENABLED is True

    settings.set("CATEGORY_PATH_ENABLED", False)
    assert settings.CATEGORY_PATH_ENABLED is False

    assert djpress_tags.post_category_link(category1, "class1") == category1.title

    # Set back to defaults
    settings.set("CATEGORY_PATH_ENABLED", True)


@pytest.mark.django_db
def test_post_category_link_with_category_path(category1):
    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH_ENABLED is True
    assert settings.CATEGORY_PATH == "test-url-category"

    expected_output = f'<a href="/{settings.CATEGORY_PATH}/general/" title="View all posts in the General category">{category1.title}</a>'

    assert djpress_tags.post_category_link(category1) == expected_output


@pytest.mark.django_db
def test_post_category_link_with_category_path_with_one_link_class(category1):
    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH_ENABLED is True
    assert settings.CATEGORY_PATH == "test-url-category"

    expected_output = f'<a href="/{settings.CATEGORY_PATH}/general/" title="View all posts in the General category" class="class1">{category1.title}</a>'

    assert djpress_tags.post_category_link(category1, "class1") == expected_output


@pytest.mark.django_db
def test_post_category_link_with_category_path_with_two_link_classes(category1):
    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH_ENABLED is True
    assert settings.CATEGORY_PATH == "test-url-category"

    expected_output = f'<a href="/{settings.CATEGORY_PATH}/general/" title="View all posts in the General category" class="class1 class2">{category1.title}</a>'

    assert (
        djpress_tags.post_category_link(category1, "class1 class2") == expected_output
    )


def test_post_date_no_post():
    context = Context({"foo": "bar"})

    assert djpress_tags.post_date(context) == ""
    assert type(djpress_tags.post_date(context)) == str


@pytest.mark.django_db
def test_post_date_with_date_archives_disabled(test_post1):
    """djpress_tags.post_date is not impacted by the DATE_ARCHIVES_ENABLED setting."""
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.DATE_ARCHIVES_ENABLED is True

    settings.set("DATE_ARCHIVES_ENABLED", False)
    assert settings.DATE_ARCHIVES_ENABLED is False

    expected_output = test_post1.date.strftime("%b %-d, %Y")

    assert djpress_tags.post_date(context) == expected_output

    # Set back to defaults
    settings.set("DATE_ARCHIVES_ENABLED", True)


@pytest.mark.django_db
def test_post_date_with_date_archives_enabled(test_post1):
    """djpress_tags.post_date is not impacted by the DATE_ARCHIVES_ENABLED setting."""
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.DATE_ARCHIVES_ENABLED is True

    expected_output = test_post1.date.strftime("%b %-d, %Y")

    assert djpress_tags.post_date(context) == expected_output


def test_post_date_link_no_post():
    context = Context({"foo": "bar"})

    assert djpress_tags.post_date_link(context) == ""
    assert type(djpress_tags.post_date_link(context)) == str


@pytest.mark.django_db
def test_post_date_link_with_date_archives_disabled(test_post1):
    """Should return just the date."""
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.DATE_ARCHIVES_ENABLED is True

    settings.set("DATE_ARCHIVES_ENABLED", False)
    assert settings.DATE_ARCHIVES_ENABLED is False

    expected_output = test_post1.date.strftime("%b %-d, %Y")

    assert djpress_tags.post_date_link(context) == expected_output

    # Set back to defaults
    settings.set("DATE_ARCHIVES_ENABLED", True)


@pytest.mark.django_db
def test_post_date_link_with_date_archives_enabled(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.DATE_ARCHIVES_ENABLED is True

    post_date = test_post1.date
    post_year = post_date.strftime("%Y")
    post_month = post_date.strftime("%m")
    post_month_name = post_date.strftime("%b")
    post_day = post_date.strftime("%d")
    post_day_name = post_date.strftime("%-d")
    post_time = post_date.strftime("%-I:%M %p")

    expected_output = (
        f'<a href="/archives/{post_year}/{post_month}/" title="View all posts in {post_month_name} {post_year}">{post_month_name}</a> '
        f'<a href="/archives/{post_year}/{post_month}/{post_day}/" title="View all posts on {post_day_name} {post_month_name} {post_year}">{post_day_name}</a>, '
        f'<a href="/archives/{post_year}/" title="View all posts in {post_year}">{post_year}</a>, '
        f"{post_time}."
    )

    assert djpress_tags.post_date_link(context) == expected_output


@pytest.mark.django_db
def test_post_date_link_with_date_archives_enabled_with_one_link_class(
    test_post1,
):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.DATE_ARCHIVES_ENABLED is True

    post_date = test_post1.date
    post_year = post_date.strftime("%Y")
    post_month = post_date.strftime("%m")
    post_month_name = post_date.strftime("%b")
    post_day = post_date.strftime("%d")
    post_day_name = post_date.strftime("%-d")
    post_time = post_date.strftime("%-I:%M %p")

    expected_output = (
        f'<a href="/archives/{post_year}/{post_month}/" title="View all posts in {post_month_name} {post_year}" class="class1">{post_month_name}</a> '
        f'<a href="/archives/{post_year}/{post_month}/{post_day}/" title="View all posts on {post_day_name} {post_month_name} {post_year}" class="class1">{post_day_name}</a>, '
        f'<a href="/archives/{post_year}/" title="View all posts in {post_year}" class="class1">{post_year}</a>, '
        f"{post_time}."
    )

    assert djpress_tags.post_date_link(context, "class1") == expected_output


@pytest.mark.django_db
def test_post_date_link_with_date_archives_enabled_with_two_link_classes(
    test_post1,
):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.DATE_ARCHIVES_ENABLED is True

    post_date = test_post1.date
    post_year = post_date.strftime("%Y")
    post_month = post_date.strftime("%m")
    post_month_name = post_date.strftime("%b")
    post_day = post_date.strftime("%d")
    post_day_name = post_date.strftime("%-d")
    post_time = post_date.strftime("%-I:%M %p")

    expected_output = (
        f'<a href="/archives/{post_year}/{post_month}/" title="View all posts in {post_month_name} {post_year}" class="class1 class2">{post_month_name}</a> '
        f'<a href="/archives/{post_year}/{post_month}/{post_day}/" title="View all posts on {post_day_name} {post_month_name} {post_year}" class="class1 class2">{post_day_name}</a>, '
        f'<a href="/archives/{post_year}/" title="View all posts in {post_year}" class="class1 class2">{post_year}</a>, '
        f"{post_time}."
    )

    assert djpress_tags.post_date_link(context, "class1 class2") == expected_output


def test_post_content_no_post():
    """If there's no post in the context, return an empty string."""
    context = Context({"foo": "bar"})

    assert djpress_tags.post_content(context) == ""
    assert type(djpress_tags.post_content(context)) == str


@pytest.mark.django_db
def test_post_content_with_post(test_post1):
    """If there's a post in the context, return the post content."""
    context = Context({"post": test_post1})

    expected_output = f"{test_post1.content_markdown}"

    assert djpress_tags.post_content(context) == expected_output


@pytest.mark.django_db
def test_post_content_with_posts(test_long_post1):
    """If there's a posts in the context, return the truncated post content."""
    # Context should have both a posts and a post to simulate the for post in posts loop
    context = Context({"posts": [test_long_post1], "post": test_long_post1})

    expected_output = (
        f"{test_long_post1.truncated_content_markdown}"
        f"{post_read_more_link(test_long_post1)}"
    )

    assert djpress_tags.post_content(context) == expected_output


@pytest.mark.django_db
def test_author_name(user):
    """author_name only works if there's a author in the context."""
    context = Context({"author": user})

    # Test case 1 - no options
    expected_output = get_author_display_name(user)
    assert djpress_tags.author_name(context) == expected_output

    # Test case 2 - with options
    expected_output = f'<h1 class="title">View posts in the {get_author_display_name(user)} author</h1>'
    assert (
        djpress_tags.author_name(
            context,
            outer="h1",
            outer_class="title",
            pre_text="View posts in the ",
            post_text=" author",
        )
        == expected_output
    )


def test_author_name_no_author():
    """If there's no author in the context, return an empty string."""
    context = Context({"foo": "bar"})

    assert djpress_tags.author_name(context) == ""
    assert type(djpress_tags.author_name(context)) == str


@pytest.mark.django_db
def test_category_title(category1):
    """category_title only works if there's a category in the context."""
    context = Context({"category": category1})

    # Test case 1 - no options
    expected_output = category1.title
    assert djpress_tags.category_title(context) == expected_output

    # Test case 2 - with options
    expected_output = (
        f'<h1 class="title">View posts in the {category1.title} category</h1>'
    )
    assert (
        djpress_tags.category_title(
            context,
            outer="h1",
            outer_class="title",
            pre_text="View posts in the ",
            post_text=" category",
        )
        == expected_output
    )


def test_category_title_no_category():
    """If there's no category in the context, return an empty string."""
    context = Context({"foo": "bar"})

    assert djpress_tags.category_title(context) == ""
    assert type(djpress_tags.category_title(context)) == str


@pytest.mark.django_db
def test_post_categories(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH == "test-url-category"

    expected_output = f'<ul><li><a href="/{settings.CATEGORY_PATH}/general/" title="View all posts in the General category">General</a></li></ul>'

    assert djpress_tags.post_categories_link(context) == expected_output


def test_post_categories_none_post_context():
    context = Context({"post": None})

    expected_output = ""
    assert djpress_tags.post_categories_link(context) == expected_output


def test_post_categories_no_post_context():
    context = Context({"foo": None})

    expected_output = ""
    assert djpress_tags.post_categories_link(context) == expected_output


@pytest.mark.django_db
def test_post_categories_no_categories_context(test_post1):
    test_post1.categories.clear()
    context = Context({"post": test_post1})

    expected_output = ""
    assert djpress_tags.post_categories_link(context) == expected_output


@pytest.mark.django_db
def test_post_categories_ul(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH == "test-url-category"

    expected_output = f'<ul><li><a href="/{settings.CATEGORY_PATH}/general/" title="View all posts in the General category">General</a></li></ul>'

    assert djpress_tags.post_categories_link(context, "ul") == expected_output


@pytest.mark.django_db
def test_post_categories_ul_class1(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH == "test-url-category"

    expected_output = f'<ul><li><a href="/{settings.CATEGORY_PATH}/general/" title="View all posts in the General category" class="class1">General</a></li></ul>'

    assert (
        djpress_tags.post_categories_link(context, outer="ul", link_class="class1")
        == expected_output
    )


@pytest.mark.django_db
def test_post_categories_ul_class1_class2(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH == "test-url-category"

    expected_output = f'<ul><li><a href="/{settings.CATEGORY_PATH}/general/" title="View all posts in the General category" class="class1 class2">General</a></li></ul>'

    assert (
        djpress_tags.post_categories_link(
            context, outer="ul", link_class="class1 class2"
        )
        == expected_output
    )


@pytest.mark.django_db
def test_post_categories_div(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH == "test-url-category"

    expected_output = f'<div><a href="/{settings.CATEGORY_PATH}/general/" title="View all posts in the General category">General</a></div>'

    assert djpress_tags.post_categories_link(context, outer="div") == expected_output


@pytest.mark.django_db
def test_post_categories_div_class1(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH == "test-url-category"

    expected_output = f'<div><a href="/{settings.CATEGORY_PATH}/general/" title="View all posts in the General category" class="class1">General</a></div>'

    assert (
        djpress_tags.post_categories_link(context, outer="div", link_class="class1")
        == expected_output
    )


@pytest.mark.django_db
def test_post_categories_div_class1_class2(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH == "test-url-category"

    expected_output = f'<div><a href="/{settings.CATEGORY_PATH}/general/" title="View all posts in the General category" class="class1 class2">General</a></div>'

    assert (
        djpress_tags.post_categories_link(
            context, outer="div", link_class="class1 class2"
        )
        == expected_output
    )


@pytest.mark.django_db
def test_post_categories_span(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH == "test-url-category"

    expected_output = f'<span><a href="/{settings.CATEGORY_PATH}/general/" title="View all posts in the General category">General</a></span>'

    assert djpress_tags.post_categories_link(context, outer="span") == expected_output


@pytest.mark.django_db
def test_post_categories_span_class1(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH == "test-url-category"

    expected_output = f'<span><a href="/{settings.CATEGORY_PATH}/general/" title="View all posts in the General category" class="class1">General</a></span>'

    assert (
        djpress_tags.post_categories_link(context, outer="span", link_class="class1")
        == expected_output
    )


@pytest.mark.django_db
def test_post_categories_span_class1_class2(test_post1):
    context = Context({"post": test_post1})

    # Confirm settings are set according to settings_testing.py
    assert settings.CATEGORY_PATH == "test-url-category"

    expected_output = f'<span><a href="/{settings.CATEGORY_PATH}/general/" title="View all posts in the General category" class="class1 class2">General</a></span>'

    assert (
        djpress_tags.post_categories_link(
            context, outer="span", link_class="class1 class2"
        )
        == expected_output
    )


@pytest.mark.django_db
def test_blog_categories(category1, category2):
    categories = Category.objects.all()

    assert category1 in categories
    assert category2 in categories

    assert djpress_tags.blog_categories() == categories_html(
        categories=categories, outer="ul", outer_class="", link_class=""
    )


@pytest.mark.django_db
def test_blog_categories_no_categories():
    assert djpress_tags.blog_categories() == ""


@pytest.mark.django_db
def test_blog_pages_no_pages():
    assert djpress_tags.blog_pages() == ""


@pytest.mark.django_db
def test_blog_pages(test_page1, test_page2):
    pages = Post.page_objects.all()

    assert test_page1 in pages
    assert test_page2 in pages

    expected_output_ul = (
        f"<ul><li>{get_page_link(page=test_page1)}</li>"
        f"<li>{get_page_link(page=test_page2)}</li></ul>"
    )

    expected_output_div = (
        f"<div>{get_page_link(page=test_page1)}, {get_page_link(page=test_page2)}</div>"
    )

    expected_output_span = f"<span>{get_page_link(page=test_page1)}, {get_page_link(page=test_page2)}</span>"

    assert djpress_tags.blog_pages() == expected_output_ul
    assert djpress_tags.blog_pages(outer="div") == expected_output_div
    assert djpress_tags.blog_pages(outer="span") == expected_output_span


@pytest.mark.django_db
def test_blog_page_title(test_post1, test_page1):
    # Test case 1 - category page
    context = Context({"category": test_post1.categories.first()})
    assert test_post1.categories.first().title == djpress_tags.blog_page_title(context)

    # Test case 2 - author page
    context = Context({"author": test_post1.author})
    assert get_author_display_name(test_post1.author) == djpress_tags.blog_page_title(
        context
    )

    # Test case 3 - single post
    context = Context({"post": test_post1})
    assert test_post1.title == djpress_tags.blog_page_title(context)

    # Test case 4 - single page
    context = Context({"post": test_page1})
    assert test_page1.title == djpress_tags.blog_page_title(context)

    # Test case 5 - no context
    context = Context()
    assert "" == djpress_tags.blog_page_title(context)


@pytest.mark.django_db
def test_is_paginated(test_post1, test_post2, test_long_post1):
    # Confirm settings are set according to settings_testing.py
    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 3

    # Test case 1 - no paginator in context
    context = Context()
    assert djpress_tags.is_paginated(context) == False

    # Test case 2 - paginator in context
    posts = Paginator(
        Post.post_objects.get_published_posts(),
        settings.RECENT_PUBLISHED_POSTS_COUNT,
    )
    page = posts.get_page(number=None)
    context = Context({"posts": page})
    assert djpress_tags.is_paginated(context) == True


def test_pagination_links_no_posts():
    context = Context()

    assert djpress_tags.pagination_links(context) == ""
    assert type(djpress_tags.pagination_links(context)) == str


@pytest.mark.django_db
def test_get_pagination_range(test_post1, test_post2, test_long_post1):
    # Test case 1 - 1 page, i.e. 3 posts with 3 posts per page
    # Confirm settings are set according to settings_testing.py
    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 3
    posts = Paginator(
        Post.post_objects.get_published_posts(),
        settings.RECENT_PUBLISHED_POSTS_COUNT,
    )
    page = posts.get_page(number=None)
    context = Context({"posts": page})
    assert djpress_tags.get_pagination_range(context) == range(1, 2)

    # Test case 2 - 2 pages, i.e. 3 posts with 2 posts per page
    settings.set("RECENT_PUBLISHED_POSTS_COUNT", 2)
    posts = Paginator(
        Post.post_objects.get_published_posts(),
        settings.RECENT_PUBLISHED_POSTS_COUNT,
    )
    page = posts.get_page(number=None)
    context = Context({"posts": page})
    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 2
    assert djpress_tags.get_pagination_range(context) == range(1, 3)

    # Test case 3 - 3 pages, i.e. 3 posts with 1 posts per page
    settings.set("RECENT_PUBLISHED_POSTS_COUNT", 1)
    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 1
    posts = Paginator(
        Post.post_objects.get_published_posts(),
        settings.RECENT_PUBLISHED_POSTS_COUNT,
    )
    page = posts.get_page(number=None)
    context = Context({"posts": page})
    assert djpress_tags.get_pagination_range(context) == range(1, 4)

    # Set back to defaults
    settings.set("RECENT_PUBLISHED_POSTS_COUNT", 3)
    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 3


@pytest.mark.django_db
def test_get_pagination_current_page(test_post1, test_post2, test_long_post1):
    # Confirm settings are set according to settings_testing.py
    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 3

    settings.set("RECENT_PUBLISHED_POSTS_COUNT", 1)
    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 1

    # Test case 1 - no page number
    posts = Paginator(
        Post.post_objects.get_published_posts(),
        settings.RECENT_PUBLISHED_POSTS_COUNT,
    )
    page = posts.get_page(number=None)
    context = Context({"posts": page})
    assert djpress_tags.get_pagination_current_page(context) == 1

    # Test case 2 - page number 1
    page = posts.get_page(number=1)
    context = Context({"posts": page})
    assert djpress_tags.get_pagination_current_page(context) == 1

    # Test case 3 - page number 2
    page = posts.get_page(number=2)
    context = Context({"posts": page})
    assert djpress_tags.get_pagination_current_page(context) == 2

    # Test case 4 - page number 3
    page = posts.get_page(number=3)
    context = Context({"posts": page})
    assert djpress_tags.get_pagination_current_page(context) == 3

    # Set back to defaults
    settings.set("RECENT_PUBLISHED_POSTS_COUNT", 3)
    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 3


@pytest.mark.django_db
def test_pagination_links_one_page(test_post1, test_post2, test_long_post1):
    # Confirm settings are set according to settings_testing.py
    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 3

    posts = Paginator(
        Post.post_objects.get_published_posts(),
        settings.RECENT_PUBLISHED_POSTS_COUNT,
    )
    page = posts.get_page(number=None)

    context = Context({"posts": page})

    previous_output = ""
    current_output = (
        f'<span class="current">'
        f"Page {page.number} of {page.paginator.num_pages}"
        f"</span>"
    )
    next_output = ""

    expected_output = f'<div class="pagination">{previous_output} {current_output} {next_output}</div>'

    assert djpress_tags.pagination_links(context) == expected_output


@pytest.mark.django_db
def test_pagination_links_two_pages(test_post1, test_post2, test_long_post1):
    # Confirm settings are set according to settings_testing.py
    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 3

    settings.set("RECENT_PUBLISHED_POSTS_COUNT", 2)

    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 2

    posts = Paginator(
        Post.post_objects.get_published_posts(),
        settings.RECENT_PUBLISHED_POSTS_COUNT,
    )

    # Test case 1 - first page with no page page
    page = posts.get_page(number=None)

    context = Context({"posts": page})

    previous_output = ""
    current_output = (
        f'<span class="current">'
        f"Page {page.number} of {page.paginator.num_pages}"
        f"</span>"
    )
    next_output = (
        f'<span class="next">'
        f'<a href="?page={page.next_page_number()}">next</a> '
        f'<a href="?page={page.paginator.num_pages}">last &raquo;</a>'
        f"</span>"
    )

    expected_output = f'<div class="pagination">{previous_output} {current_output} {next_output}</div>'

    assert djpress_tags.pagination_links(context) == expected_output

    # Test case 2 - first page with page number 1
    page = posts.get_page(number=1)

    context = Context({"posts": page})

    previous_output = ""
    current_output = (
        f'<span class="current">'
        f"Page {page.number} of {page.paginator.num_pages}"
        f"</span>"
    )
    next_output = (
        f'<span class="next">'
        f'<a href="?page={page.next_page_number()}">next</a> '
        f'<a href="?page={page.paginator.num_pages}">last &raquo;</a>'
        f"</span>"
    )

    expected_output = f'<div class="pagination">{previous_output} {current_output} {next_output}</div>'

    assert djpress_tags.pagination_links(context) == expected_output

    # Test case 3 - second page with page number 2
    page = posts.get_page(number=2)

    context = Context({"posts": page})

    previous_output = (
        f'<span class="previous">'
        f'<a href="?page=1">&laquo; first</a> '
        f'<a href="?page={page.previous_page_number()}">previous</a>'
        f"</span>"
    )
    current_output = (
        f'<span class="current">'
        f"Page {page.number} of {page.paginator.num_pages}"
        f"</span>"
    )
    next_output = ""

    expected_output = f'<div class="pagination">{previous_output} {current_output} {next_output}</div>'

    assert djpress_tags.pagination_links(context) == expected_output

    # Set back to defaults
    settings.set("RECENT_PUBLISHED_POSTS_COUNT", 3)
    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 3


@pytest.mark.django_db
def test_pagination_links_three_pages(test_post1, test_post2, test_long_post1):
    # Confirm settings are set according to settings_testing.py
    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 3

    settings.set("RECENT_PUBLISHED_POSTS_COUNT", 1)

    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 1

    posts = Paginator(
        Post.post_objects.get_published_posts(),
        settings.RECENT_PUBLISHED_POSTS_COUNT,
    )

    # Test case 1 - first page with no page number
    page = posts.get_page(number=None)

    context = Context({"posts": page})

    previous_output = ""
    current_output = (
        f'<span class="current">'
        f"Page {page.number} of {page.paginator.num_pages}"
        f"</span>"
    )
    next_output = (
        f'<span class="next">'
        f'<a href="?page={page.next_page_number()}">next</a> '
        f'<a href="?page={page.paginator.num_pages}">last &raquo;</a>'
        f"</span>"
    )

    expected_output = f'<div class="pagination">{previous_output} {current_output} {next_output}</div>'

    assert djpress_tags.pagination_links(context) == expected_output

    # Test case 2 - first page with page number 1
    page = posts.get_page(number=1)

    context = Context({"posts": page})

    previous_output = ""
    current_output = (
        f'<span class="current">'
        f"Page {page.number} of {page.paginator.num_pages}"
        f"</span>"
    )
    next_output = (
        f'<span class="next">'
        f'<a href="?page={page.next_page_number()}">next</a> '
        f'<a href="?page={page.paginator.num_pages}">last &raquo;</a>'
        f"</span>"
    )

    expected_output = f'<div class="pagination">{previous_output} {current_output} {next_output}</div>'

    assert djpress_tags.pagination_links(context) == expected_output

    # Test case 3 - first page with page number 2
    page = posts.get_page(number=2)

    context = Context({"posts": page})

    previous_output = (
        f'<span class="previous">'
        f'<a href="?page=1">&laquo; first</a> '
        f'<a href="?page={page.previous_page_number()}">previous</a>'
        f"</span>"
    )
    current_output = (
        f'<span class="current">'
        f"Page {page.number} of {page.paginator.num_pages}"
        f"</span>"
    )
    next_output = (
        f'<span class="next">'
        f'<a href="?page={page.next_page_number()}">next</a> '
        f'<a href="?page={page.paginator.num_pages}">last &raquo;</a>'
        f"</span>"
    )

    expected_output = f'<div class="pagination">{previous_output} {current_output} {next_output}</div>'

    assert djpress_tags.pagination_links(context) == expected_output

    # Test case 4 - first page with page number 3
    page = posts.get_page(number=3)

    context = Context({"posts": page})

    previous_output = (
        f'<span class="previous">'
        f'<a href="?page=1">&laquo; first</a> '
        f'<a href="?page={page.previous_page_number()}">previous</a>'
        f"</span>"
    )
    current_output = (
        f'<span class="current">'
        f"Page {page.number} of {page.paginator.num_pages}"
        f"</span>"
    )
    next_output = ""

    expected_output = f'<div class="pagination">{previous_output} {current_output} {next_output}</div>'

    assert djpress_tags.pagination_links(context) == expected_output

    # Set back to defaults
    settings.set("RECENT_PUBLISHED_POSTS_COUNT", 3)
    assert settings.RECENT_PUBLISHED_POSTS_COUNT == 3


@pytest.mark.django_db
def test_page_link(test_page1):
    # Test 1 - page with a non existent page_slug
    page_slug = ""
    assert djpress_tags.page_link(page_slug=page_slug) == ""

    # Test 2 - page with a page_slug and no options
    page_slug = test_page1.slug
    outer = ""
    outer_class = ""
    link_class = ""

    output = get_page_link(page=test_page1, link_class=link_class)
    expected_output = f"<div{outer_class}>{output}</div>"
    assert djpress_tags.page_link(page_slug=page_slug) == expected_output

    # Test 3 - page with a page_slug and div and no options
    page_slug = test_page1.slug
    outer = "div"
    outer_class = ""
    link_class = ""

    output = get_page_link(page=test_page1, link_class=link_class)
    expected_output = f"<div{outer_class}>{output}</div>"
    assert djpress_tags.page_link(page_slug=page_slug) == expected_output

    # Test 4 - page with a page_slug and div and all options
    page_slug = test_page1.slug
    outer = "div"
    outer_class = "outerclass"
    link_class = "linkclass"

    outer_class = f' class="{outer_class}"' if outer_class else ""
    output = get_page_link(page=test_page1, link_class=link_class)
    expected_output = f"<div{outer_class}>{output}</div>"
    assert (
        djpress_tags.page_link(
            page_slug=page_slug,
            outer=outer,
            outer_class=outer_class,
            link_class=link_class,
        )
        == expected_output
    )

    # Test 5 - page with a page_slug and span and all options
    page_slug = test_page1.slug
    outer = "span"
    outer_class = "outerclass"
    link_class = "linkclass"

    outer_class = f' class="{outer_class}"' if outer_class else ""
    output = get_page_link(page=test_page1, link_class=link_class)
    expected_output = f"<span{outer_class}>{output}</span>"
    assert (
        djpress_tags.page_link(
            page_slug=page_slug,
            outer=outer,
            outer_class=outer_class,
            link_class=link_class,
        )
        == expected_output
    )

    # Test 6 - page with a page_slug and li and no options
    page_slug = test_page1.slug
    outer = "li"
    outer_class = ""
    link_class = ""

    outer_class = f' class="{outer_class}"' if outer_class else ""
    output = get_page_link(page=test_page1, link_class=link_class)
    expected_output = f"<li{outer_class}>{output}</li>"
    assert (
        djpress_tags.page_link(
            page_slug=page_slug,
            outer=outer,
            outer_class=outer_class,
            link_class=link_class,
        )
        == expected_output
    )

    # Test 6 - page with a page_slug and a wrong outer and no options
    page_slug = test_page1.slug
    outer = "foobar"
    outer_class = ""
    link_class = ""

    outer_class = f' class="{outer_class}"' if outer_class else ""
    output = get_page_link(page=test_page1, link_class=link_class)
    expected_output = f"{output}"
    assert (
        djpress_tags.page_link(
            page_slug=page_slug,
            outer=outer,
            outer_class=outer_class,
            link_class=link_class,
        )
        == expected_output
    )



"""DJ Press views file.

There are two type of views - those that return a collection of posts, and then a view
that returns a single post.
"""

import logging

from django.contrib.auth.models import User
from django.core.paginator import Paginator
from django.http import Http404, HttpRequest, HttpResponse, HttpResponseBadRequest
from django.shortcuts import render

from djpress.conf import settings
from djpress.models import Category, Post
from djpress.utils import get_template_name, validate_date

logger = logging.getLogger(__name__)


def index(
    request: HttpRequest,
) -> HttpResponse:
    """View for the index page.

    Args:
        request (HttpRequest): The request object.

    Returns:
        HttpResponse: The response.

    Context:
        posts (Page): The published posts as a Page object.
    """
    template_names: list[str] = [
        "djpress/home.html",
        "djpress/index.html",
    ]

    template: str = get_template_name(templates=template_names)

    posts = Paginator(
        Post.post_objects.get_published_posts(),
        settings.RECENT_PUBLISHED_POSTS_COUNT,
    )
    page_number = request.GET.get("page")
    page = posts.get_page(page_number)

    return render(
        request=request,
        template_name=template,
        context={"posts": page},
    )


def archives_posts(
    request: HttpRequest,
    year: str,
    month: str = "",
    day: str = "",
) -> HttpResponse:
    """View for the date-based archives pages.

    Args:
        request (HttpRequest): The request object.
        year (str): The year.
        month (str): The month.
        day (str): The day.

    Returns:
        HttpResponse: The response.

    Context:
        posts (Page): The published posts for the date as a Page object.
    """
    template_names: list[str] = [
        "djpress/archives.html",
        "djpress/index.html",
    ]
    template: str = get_template_name(templates=template_names)

    try:
        validate_date(year, month, day)
    except ValueError:
        msg = "Invalid date"
        return HttpResponseBadRequest(msg)

    published_posts = Post.post_objects.get_published_posts()

    # Django converts strings to integers when they are passed to the filter
    if day:
        filtered_posts = published_posts.filter(
            date__year=year,
            date__month=month,
            date__day=day,
        )

    elif month:
        filtered_posts = published_posts.filter(
            date__year=year,
            date__month=month,
        )

    elif year:
        filtered_posts = published_posts.filter(
            date__year=year,
        )

    posts = Paginator(filtered_posts, settings.RECENT_PUBLISHED_POSTS_COUNT)
    page_number = request.GET.get("page")
    page = posts.get_page(page_number)

    return render(
        request=request,
        template_name=template,
        context={"posts": page},
    )


def category_posts(request: HttpRequest, slug: str) -> HttpResponse:
    """View for posts by category.

    Args:
        request (HttpRequest): The request object.
        slug (str): The category slug.

    Returns:
        HttpResponse: The response.

    Context:
        posts (Page): The published posts for the category as a Page object.
        category (Category): The category object.
    """
    template_names: list[str] = [
        "djpress/category.html",
        "djpress/index.html",
    ]
    template: str = get_template_name(templates=template_names)

    try:
        category: Category = Category.objects.get_category_by_slug(slug=slug)
    except ValueError as exc:
        msg = "Category not found"
        raise Http404(msg) from exc

    posts = Paginator(
        Post.post_objects.get_published_posts_by_category(category),
        settings.RECENT_PUBLISHED_POSTS_COUNT,
    )
    page_number = request.GET.get("page")
    page = posts.get_page(page_number)

    return render(
        request=request,
        template_name=template,
        context={"posts": page, "category": category},
    )


def author_posts(request: HttpRequest, author: str) -> HttpResponse:
    """View for posts by author.

    Args:
        request (HttpRequest): The request object.
        author (str): The author username.

    Returns:
        HttpResponse: The response.

    Context:
        posts (Page): The published posts by the author as a Page object.
        author (User): The author as a User object.
    """
    template_names: list[str] = [
        "djpress/author.html",
        "djpress/index.html",
    ]
    template: str = get_template_name(templates=template_names)

    try:
        user: User = User.objects.get(username=author)
    except User.DoesNotExist as exc:
        msg = "Author not found"
        raise Http404(msg) from exc

    posts = Paginator(
        Post.post_objects.get_published_posts_by_author(user),
        settings.RECENT_PUBLISHED_POSTS_COUNT,
    )
    page_number = request.GET.get("page")
    page = posts.get_page(page_number)

    return render(
        request=request,
        template_name=template,
        context={"posts": page, "author": user},
    )


def post_detail(request: HttpRequest, path: str) -> HttpResponse:
    """View for a single post.

    Args:
        request (HttpRequest): The request object.
        path (str): The path to the post.

    Returns:
        HttpResponse: The response.

    Context:
        post (Post): The post object.
    """
    template_names: list[str] = [
        "djpress/single.html",
        "djpress/index.html",
    ]

    try:
        page: Post = Post.page_objects.get_published_page_by_path(path)
        context: dict = {"post": page}
        # If the page is found, use the page template
        template_names.insert(0, "djpress/page.html")
    except ValueError:
        try:
            post = Post.post_objects.get_published_post_by_path(path)
            context: dict = {"post": post}
        except ValueError as exc:
            msg = "Post not found"
            raise Http404(msg) from exc

    template: str = get_template_name(templates=template_names)
    return render(
        request=request,
        context=context,
        template_name=template,
    )



"""Configuration settings for DJ Press."""

from typing import Any

from django.conf import settings as django_settings
from django.core.cache import cache

from djpress import app_settings as default_settings


class Settings:
    """Class to manage DJ Press settings."""

    def __init__(
        self: "Settings",
        default_settings_module: object,
        user_settings_module: object,
    ) -> None:
        """Initialize the settings object."""
        self._default_settings = default_settings_module
        self._user_settings = user_settings_module

    def __getattr__(self: "Settings", name: str) -> Any:  # noqa: ANN401
        """Get the value of a setting.

        Args:
            name (str): The name of the setting to get.

        Returns:
            Any: The value of the setting.

        Raises:
            AttributeError: If the setting is not found.
        """
        # If the setting is found in the user settings, return it
        if hasattr(self._user_settings, name):
            return getattr(self._user_settings, name)
        # If the setting is found in the default settings, return it
        if hasattr(self._default_settings, name):
            return getattr(self._default_settings, name)
        # If the setting is not found in either, raise an AttributeError
        msg = f"Setting '{name}' not found"
        raise AttributeError(msg)

    def set(self: "Settings", name: str, value: object) -> None:
        """Set the value of a setting and invalidate the cache if necessary."""
        if self._should_invalidate_cache():
            cache.clear()
        setattr(self._user_settings, name, value)

    def _should_invalidate_cache(self: "Settings") -> bool:
        """Check if cache should be invalidated based on cache settings."""
        return self.cache_categories or self.cache_recent_published_posts

    @property
    def cache_categories(self: "Settings") -> bool:
        """Return the value of the CACHE_CATEGORIES setting."""
        return getattr(
            self._user_settings,
            "CACHE_CATEGORIES",
            getattr(self._default_settings, "CACHE_CATEGORIES", True),
        )

    @property
    def cache_recent_published_posts(self: "Settings") -> bool:
        """Return the value of the CACHE_RECENT_PUBLISHED_POSTS setting."""
        return getattr(
            self._user_settings,
            "CACHE_RECENT_PUBLISHED_POSTS",
            getattr(self._default_settings, "CACHE_RECENT_PUBLISHED_POSTS", False),
        )


settings = Settings(default_settings, django_settings)



"""Djpress app configuration."""

from django.apps import AppConfig


class DjpressConfig(AppConfig):
    """Djpress app configuration."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "djpress"
    label = "djpress"

    def ready(self: "DjpressConfig") -> None:
        """Import signals."""
        import djpress.signals  # noqa: F401
